from requests.exceptions import HTTPError, ConnectionError

from buycoins.exceptions import ClientError


def check_response(response, exception):
    """Checks the request"s response to raise any possible exception.

    Args:
        response (): Response object
        exception (): Exception class to be raised: WalletError, P2PError, NGNTError

    Returns:
        None

    """

    if not exception:
        exception = Exception

    if type(response) == ConnectionError:
        message = "{}: Failed to establish a connection".format(response.__doc__)
        raise ClientError(message, 404)
    elif type(response) == HTTPError:
        status_code = response.response.status_code
        if str(status_code).startswith('4'):
            json_response = response.response.json()
            error_message = json_response['errors']
            raise ClientError(error_message, status_code)
        else:
            error_message = response.__doc__
            raise ClientError(error_message, status_code)
    elif "errors" in response:
        error = response["errors"][0]
        error_message = error["message"]
        raise exception(error_message, 404)



from buycoins.client import BuyCoinsClient
from buycoins.exceptions import AccountError, ClientError
from buycoins.utils import check_response


class NGNT(BuyCoinsClient):
    """The NGNT class handles the generations of virtual bank deposit account.
    """

    def createDepositAccount(self, accountName: str):
        """Creates a virtual deposit account under the supplied name.

        Args:
            accountName (str): Name of the new virtual deposit account to be generated*.

        Returns:
            response: A JSON object containing the response from the request.

        """
        try:
            if not accountName:
                raise AccountError("Invalid account name passed", 404)

            self.accountName = accountName

            __variables = {
                "accountName": self.accountName
            }

            self.__query = """
                mutation createDepositAccount($accountName: String!) {
                    createDepositAccount(accountName: $accountName) {
                        accountNumber
                        accountName
                        accountType
                        bankName
                        accountReference
                  }
                }
            """
            response = self._execute_request(query=self.__query, variables=__variables)
            check_response(response, AccountError)
        except (AccountError, ClientError) as e:
            return e.response
        else:
            return response


from buycoins.client import BuyCoinsClient
from buycoins.exceptions import WalletError, ClientError
from buycoins.p2p import P2P
from buycoins.utils import check_response


class Wallet(BuyCoinsClient):
    """The Wallet class handles the buying, selling, buy transactions and the generation of wallet address 
    for receiving cryptocurrencies.
    """

    supported_cryptocurrencies = ["bitcoin", "ethereum", "litecoin", "naira_token", "usd_coin", "usd_tether"]
    status = ["open", "completed"]

    def buyCrypto(self, currency: str = "bitcoin", coin_amount: float = 0.01):
        """Buys a cryptocurrency, for the given amount passed.

        Args:
            currency (str): The cryptocurrency to be bought.
            coin_amount(float): Amount of currency to be bought.

        Returns:
            response: A JSON object containing response from the request.
        """
  
        try:
            if currency not in self.supported_cryptocurrencies:
                raise WalletError("Invalid or unsupported cryptocurrency", 404)

            p2p = P2P()
            price_info = p2p.getCurrentPrice(orderSide="buy", currency=currency)
            price_id = price_info[0]["id"]
            self.__query = """
                mutation BuyCoin($price: ID!, $coin_amount: BigDecimal!, $currency: Cryptocurrency){
                        buy(price: $price, coin_amount: $coin_amount, cryptocurrency: $currency) {
                            id
                            cryptocurrency
                            status
                            totalCoinAmount
                            side
                        }
                    }
                """

            __variables = {
                "price": price_id,
                "coin_amount": coin_amount,
                "currency": currency
            }
            response = self._execute_request(query=self.__query, variables=__variables)
            check_response(response, WalletError)
        except (WalletError, ClientError) as e:
            return e.response
        else:
            return response["data"]["buy"]

    def sellCrypto(self, currency: str = "bitcoin", coin_amount: float = 0.01):
        """Sells a cryptocurrency, for the given amount passed.

        Args:
            currency (str): The cryptocurrency to be sold.
            coin_amount(float): Amount of currency to be bought.

        Returns:
            response: A JSON object containing response from the request.
        """

        try:
            if currency not in self.supported_cryptocurrencies:
                raise WalletError("Invalid or unsupported cryptocurrency", 404)

            p2p = P2P()
            price_info = p2p.getCurrentPrice(orderSide="sell", currency=currency)
            price_id = price_info[0]["id"]
            self.__query = """
                mutation SellCoin($price: ID!, $coin_amount: BigDecimal!, $currency: Cryptocurrency){
                        sell(price: $price, coin_amount: $coin_amount, cryptocurrency: $currency) {
                            id
                            cryptocurrency
                            status
                            totalCoinAmount
                            side
                        }
                    }
                """

            __variables = {
                "price": price_id,
                "coin_amount": coin_amount,
                "currency": currency
            }

            response = self._execute_request(query=self.__query, variables=__variables)
            check_response(response, WalletError)
        except (WalletError, ClientError) as e:
            return e.response
        else:
            return response["data"]["sell"]

    def getNetworkFee(self, currency: str = "bitcoin", coin_amount: float = 0.01):
        """Retrieves NetworkFee for the supplied cryptocurrency.

        Args:
            currency (str): The cryptocurrency whose network fee is been checked.
            coin_amount(float): Amount of currency.

        Returns:
            response: A JSON object containing response from the request.
        """

        try:
            if currency not in self.supported_cryptocurrencies:
                raise WalletError("Invalid or unsupported cryptocurrency", 404)

            self.__query = """
                query NetworkFee($currency: Cryptocurrency, $amount: BigDecimal!) {
                    getEstimatedNetworkFee(cryptocurrency: $currency, amount: $amount) {
                        estimatedFee
                        total
                    }
                }
            """

            __variables = {
                "currency": currency,
                "amount": coin_amount
            }

            response = self._execute_request(query=self.__query, variables=__variables)
            check_response(response, WalletError)
        except (WalletError, ClientError) as e:
            return e.response
        else:
            return response["data"]["getEstimatedNetworkFee"]

    def createAddress(self, currency: str = "bitcoin"):
        """Creates a wallet address for the supplied cryptocurrency.

        Args:
            currency (str): The cryptocurrency wallet address to be created.

        Returns:
            response: A JSON object containing response from the request.
        """

        try:
            if currency not in self.supported_cryptocurrencies:
                raise WalletError("Invalid or unsupported cryptocurrency", 404)

            self.__query = """
                mutation CreateWalletAddress($currency: Cryptocurrency) {
                    createAddress(cryptocurrency: $currency) {
                        cryptocurrency
                        address
                    }
                }
            """

            __variables = {
                "currency": currency
            }

            response = self._execute_request(query=self.__query, variables=__variables)
            check_response(response, WalletError)
        except (WalletError, ClientError) as e:
            return e.response
        else:
            return response["data"]["createAddress"]

    def sendCrypto(self, address: str, currency: str = "bitcoin", coin_amount: float = 0.01):
        """Sells a cryptocurrency, for the given amount passed.

        Args:
            currency (str): The cryptocurrency to be sold.
            coin_amount(float): Amount of currency to be bought.

        Returns:
            response: A JSON object containing response from the request.
        """

        try:
            if currency not in self.supported_cryptocurrencies:
                raise WalletError("Invalid or unsupported cryptocurrency", 404)

            if not address:
                raise WalletError("Invalid address", 404)

            self.__query = """
                mutation SendCrypto($amount: BigDecimal!, $currency: Cryptocurrency, $address: String!){
                    send(cryptocurrency: $currency, amount: $amount, address: $address) {
                        id
                        address
                        amount
                        cryptocurrency
                        fee
                        status
                        transaction {
                            txhash
                            id
                        }
                    }
                }
                """

            __variables = {
                "address": address,
                "amount": coin_amount,
                "currency": currency
            }

            response = self._execute_request(query=self.__query, variables=__variables)
            check_response(response, WalletError)
        except (WalletError, ClientError) as e:
            return e.response
        else:
            return response["data"]["SendCoin"]

    def getBalances(self):
        """Retrieves user cryptocurrency balances

        Returns:
            response: A JSON object containing the user cryptocurrency balances.

        """

        try:

            self.__query = """
                query {
                    getBalances {
                        id
                        cryptocurrency
                        confirmedBalance
                    }
                }
            """
            response = self._execute_request(query=self.__query)
            check_response(response, WalletError)
        except (WalletError, ClientError) as e:
            return e.response
        else:
            return response["data"]["getBalances"]



from buycoins.client import BuyCoinsClient
from buycoins.exceptions import P2PError, ClientError
from buycoins.utils import check_response


class P2P(BuyCoinsClient):
    """The P2P class handles peer-2-peer transactions.
    """

    supported_cryptocurrencies = ["bitcoin", "ethereum", "litecoin", "naira_token", "usd_coin", "usd_tether"]
    side = ["buy", "sell"]
    status = ["open", "completed"]

    def getPrices(self):
        """Returns the current price for supported cryptocurrencies on BuyCoins

        Returns:
            response: An array of cryptocurrency data.

        """

        try:
            self.__query = """
                query {
                  getPrices {
                    id
                    cryptocurrency
                    buyPricePerCoin
                    minBuy
                    maxBuy
                    expiresAt
                  }
                }    
            """

            response = self._execute_request(query=self.__query)
            check_response(response, P2PError)
        except (P2PError, ClientError) as e:
            return e.response
        else:
            return response['data']['getPrices']

    def getCurrentPrice(self, orderSide: str = "buy", currency: str = "bitcoin"):
        """Retrieves the current `side` price for the supplied cryptocurrency.

        Args:
            orderSide (str):  The order side which can either be buy or sell.
            currency (str): The cryptocurrency whose current price is to be retrieved.

        Returns:
            response: A JSON object containing response from the request.
        """
        try:
            if currency not in self.supported_cryptocurrencies:
                raise P2PError("Invalid or unsupported cryptocurrency", 404)

            if orderSide not in self.side:
                raise P2PError("Invalid order side", 404)

            self.__query = """
                query GetBuyCoinsPrices($side: OrderSide, $currency: Cryptocurrency) {
                  getPrices(side: $side, cryptocurrency: $currency){
                    buyPricePerCoin
                    cryptocurrency
                    id
                    maxBuy
                    maxSell
                    minBuy
                    minCoinAmount
                    minSell
                    sellPricePerCoin
                    status
                  }
                }
            """

            __variables = {
                "side": orderSide,
                "currency": currency
            }

            response = self._execute_request(query=self.__query, variables=__variables)
            check_response(response, P2PError)
        except (P2PError, ClientError) as e:
            return e.response
        else:
            return response["data"]["getPrices"]

    def getDynamicPriceExpiry(self, status: str = "open", side: str = "buy", currency: str = "bitcoin"):
        """Retrieves the dynamic prices for available cryptocurrencies.

        Args:
            status (str): The status of the current order.

        Returns:
            response: A JSON object containing the response from the request.

        """

        try:

            if status not in self.status:
                raise P2PError("Invalid status passed", 404)

            if side not in self.side:
                raise P2PError("Invalid side passed", 404)

            if currency not in self.supported_cryptocurrencies:
                raise P2PError("Invalid or unsupported cryptocurrency", 404)

            self.__query = """
                query GetOrders($status: GetOrdersStatus!){
                    getOrders(status: $status) {
                        dynamicPriceExpiry    
                    }
                }
            """

            __variables = {
                "status": status,
            }

            response = self._execute_request(query=self.__query, variables=__variables)
            check_response(response, P2PError)
        except (P2PError, ClientError) as e:
            return e.response
        else:
            return response["data"]["getOrders"]

    def placeLimitOrder(self, orderSide: str = "buy", coinAmount: float = 0.01, currency: str = "bitcoin",
                        staticPrice: int = 100000, priceType: str = "static"):
        """Places limit order for the supplied cryptocurrency.

        Args:
            orderSide (str): The side of the order. This could be `buy` or `sell`.
            coinAmount (str): The amount the limit order is based on.
            currency (str): The cryptocurrency involved in the limit order.
            staticPrice (str, optional): Static price for the cryptocurrency in Naira.
            priceType (str): Static or dynamic price for the cryptocurrency.

        Returns:
            response: A JSON object containing the result from the request.

        """

        try:
            if orderSide not in self.side:
                raise P2PError("Invalid side passed", 404)

            if currency not in self.supported_cryptocurrencies:
                raise P2PError("Invalid or unsupported cryptocurrency", 404)

            self.__query = """
                mutation PostLimitOrder($orderSide: OrderSide!, $coinAmount: BigDecimal!, $cryptocurrency: Cryptocurrency, $staticPrice: BigDecimal, $priceType: PriceType!){
                    postLimitOrder(orderSide: $orderSide, coinAmount: $coinAmount, cryptocurrency: $cryptocurrency, staticPrice: $staticPrice, priceType: $priceType) {
                        id
                        cryptocurrency
                        coinAmount
                        side
                        status 
                        createdAt
                        pricePerCoin
                        priceType
                        staticPrice
                        dynamicExchangeRate
                    }
                }
           """

            __variables = {
                "orderSide": orderSide,
                "coinAmount": coinAmount,
                "cryptocurrency": currency,
                "staticPrice": staticPrice,
                "priceType": priceType
            }

            response = self._execute_request(query=self.__query, variables=__variables)
            check_response(response, P2PError)
        except (P2PError, ClientError) as e:
            return e.response
        else:
            return response["data"]["postLimitOrder"]

    def postMarketOrder(self, orderSide: str = "buy", coinAmount: float = 0.01, currency: str = "bitcoin"):
        """Posts a market order for the supplied cryptocurrency.

        Args:
            orderSide (str): The type of order to be placed. It could either be `buy` or `sell`.
            coinAmount (float): Amount of coin to be sold.
            currency (str): Cryptocurrency involved in the market order.

        Returns:
            response: A JSON object containing the response from the request.

        """

        try:
            if orderSide not in self.side:
                raise P2PError("Invalid side passed", 404)

            if currency not in self.supported_cryptocurrencies:
                raise P2PError("Invalid or unsupported cryptocurrency", 404)

            self.__query = """
                mutation PostMarketOrder($orderSide: OrderSide!, $coinAmount: BigDecimal!, $cryptocurrency: Cryptocurrency){
                    postMarketOrder(orderSide: $orderSide, coinAmount: $coinAmount, cryptocurrency: $cryptocurrency){
                        id
                        cryptocurrency
                        coinAmount
                        side
                        status 
                        createdAt
                        pricePerCoin
                        priceType
                        staticPrice
                        dynamicExchangeRate
                    }
                }
            """

            __variables = {
                "orderSide": orderSide,
                "coinAmount": coinAmount,
                "cryptocurrency": currency,
            }

            response = self._execute_request(query=self.__query, variables=__variables)
            check_response(response, P2PError)
        except (P2PError, ClientError) as e:
            return e.response
        else:
            return response["data"]["postMarketOrder"]

    def getOrders(self, status: str = "open"):
        """Retrieves orders based on their status.

        Args:
            status (str): Status of the order which could either be `open` or `completed`.

        Returns:
            response: A JSON object containing the response from the request.

        """
        try:
            if status not in self.status:
                raise P2PError("Invalid status passed", 404)

            self.__query = """
                query GetOrders($status: GetOrdersStatus!){
                    getOrders(status: $status) {
                        dynamicPriceExpiry
                        orders {
                          edges {
                            node {
                              id
                              cryptocurrency
                              coinAmount
                              side
                              status
                              createdAt
                              pricePerCoin
                              priceType
                              staticPrice
                              dynamicExchangeRate
                            }
                          }
                        }
                    }
                }
            """

            __variables = {
                "status": status
            }

            response = self._execute_request(query=self.__query, variables=__variables)
            check_response(response, P2PError)
        except (P2PError, ClientError) as e:
            return e.response
        else:
            return response["data"]["getOrders"]

    def getMarketBook(self):
        """Retrieves market history.

        Returns:
            response: A JSON object containing response from the request.

        """

        try:
            self.__query = """
                query {
                  getMarketBook {
                    dynamicPriceExpiry
                    orders {
                      edges {
                        node {
                          id
                          cryptocurrency
                          coinAmount
                          side
                          status 
                          createdAt
                          pricePerCoin
                          priceType
                          staticPrice
                          dynamicExchangeRate
                        }
                      }
                    }
                  }
                }
            """

            response = self._execute_request(query=self.__query)
            check_response(response, P2PError)
        except (P2PError, ClientError) as e:
            return e.response
        return response["data"]["getMarketBook"]



"""
The Exception classes below are child classes of Python's `Exception` class.

Returns:
    A JSON object containing the message and status code.
"""

class ClientError(Exception):
    def __init__(self, *args):
        if args:
            self.message = args[0]
            self.status_code = args[1]
        else:
            self.message = None
            self.status_code = 401

    @property
    def response(self):
        return {
            "status_code": self.status_code,
            "message": "ClientError: " + self.message
        }


class QueryError(Exception):
    def __init__(self, *args):
        if args:
            self.message = args[0]
            self.status_code = args[1]
        else:
            self.message = None
            self.status_code = 404

    @property
    def response(self):
        return {
            "status_code": self.status_code,
            "message": "QueryError: " + self.message
        }


class P2PError(Exception):
    def __init__(self, *args):
        if args:
            self.message = args[0]
            self.status_code = args[1]
        else:
            self.message = None
            self.status_code = 404

    @property
    def response(self):
        return {
            "status_code": self.status_code,
            "message": "P2PError: " + self.message
        }


class AccountError(Exception):
    def __init__(self, *args):
        if args:
            self.message = args[0]
            self.status_code = args[1]
        else:
            self.message = None
            self.status_code = 404

    @property
    def response(self):
        return {
            "status_code": self.status_code,
            "message": "AccountError: " + self.message
        }


class WalletError(Exception):
    def __init__(self, *args):
        if args:
            self.message = args[0]
            self.status_code = args[1]
        else:
            self.message = None
            self.status_code = 404

    @property
    def response(self):
        return {
            "status_code": self.status_code,
            "message": "WalletError" + self.message
        }


import pathlib
from setuptools import setup, find_packages

HERE = pathlib.Path(__file__).parent

README = (HERE / "README.md").read_text()

setup(
    name="buycoins-python",
    version="0.3.4",
    description="Python SDK for the BuyCoin\"s API",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/Youngestdev/BuyCoins-Python",
    author="Abdulazeez Abdulazeez Adeshina",
    author_email="youngestdev@gmail.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    install_requires=["python-graphql-client", "requests", "python-decouple"],
    packages=find_packages(),
    python_requires=">=3.6"
)



from decouple import config
from python_graphql_client import GraphqlClient
from requests.auth import HTTPBasicAuth
from requests.exceptions import HTTPError, ConnectionError

from buycoins.exceptions import QueryError

auth_key = config("auth_key")

class BuyCoinsClient:
    def __init__(self):
        self.__endpoint = "https://backend.buycoins.tech/api/graphql"
        self.__username = ""
        self.__password = ""
        self.__auth_key = auth_key

    def _split_auth_key(self):
        self.__username, self.__password = self.__auth_key.split(":")

    def _initiate_client(self):
        self._split_auth_key()
        try:
            self.__auth = HTTPBasicAuth(self.__username, self.__password)
            self.__client = GraphqlClient(self.__endpoint, auth=self.__auth)
        except (HTTPError, ConnectionError) as e:
            return e
        else:
            return self.__client

    def _execute_request(self, query: str, variables: dict = {}):
        if not query or query == "":
            raise QueryError("Invalid query passed!", 404)

        try:
            self._initiate_client()
            request = self.__client.execute(query=query, variables=variables)

        except HTTPError as e:
            return e
        except QueryError as e:
            return e.response
        except ConnectionError as e:
            return e
        else:
            return request



import hashlib
import hmac


class Webhook:
    """The Webhook class is responsible for verifying requests sent to your BuyCoins webhook URL.

    """

    def __init__(self, body: bytes, token: str, header_signature: str = "X-Webhook-Signature"):
        """

        Args:
            body (): request body from BuyCoins
            token (): BuyCoins generated webhook token
            header_signature (): "X-Webhook-Signature header"
        """
        self.token = token
        self.body = body
        self.header_signature = header_signature

    def verifyRequest(self):
        """Verify the supplied request.

        Returns:
            Bool: `True` if the request originated from BuyCoins or `False ` if the request didn't originate from BuyCoins
        """
        signed_key = self.token.encode("utf-8")
        if type(self.body) != bytes:
            self.body = bytes(self.body)

        supplied_value = hmac.new(signed_key, self.body, hashlib.sha1)
        return supplied_value == self.header_signature
